from datetime import datetime
from flask import render_template, flash, redirect, url_for, request
from flask_login import current_user, login_required
from app import db
from app.main.forms import EditProfileForm, EmptyForm, EditUserForm, \
    EditOrderForm, AdminResetPasswordForm, NewUserForm, EditRestaurantForm, NewRestaurantForm
from app.auth.permissions import admin_permission
from app.models import User, Order, PaymentType, Restaurant
from app.main import bp


@bp.before_app_request
def before_request():
    if current_user.is_authenticated:
        current_user.last_seen = datetime.utcnow()
        db.session.commit()


@bp.route('/', methods=['GET', 'POST'])
@bp.route('/index', methods=['GET', 'POST'])
@login_required
def index():
    return render_template('index.html', title='Home')


@bp.route('/user/<username>')
@login_required
def user(username):
    user = User.query.filter_by(username=username).first_or_404()
    form = EmptyForm()
    return render_template('user.html', user=user, form=form)


@bp.route('/edit_profile', methods=['GET', 'POST'])
@login_required
def edit_profile():
    form = EditProfileForm(current_user.username)
    if form.validate_on_submit():
        current_user.username = form.username.data
        current_user.email = form.email.data
        current_user.first_name = form.first_name.data
        current_user.last_name = form.last_name.data
        db.session.commit()
        flash('Your changes have been saved.')
        return redirect(url_for('main.edit_profile'))

    elif request.method == 'GET':
        form.username.data = current_user.username
        form.email.data = current_user.email
        form.first_name.data = current_user.first_name
        form.last_name.data = current_user.last_name

    return render_template('edit_profile.html', title='Edit Profile', form=form)


@bp.route('/admin', methods=['GET', 'POST'])
@login_required
@admin_permission.require()
def admin():
    return render_template('admin.html', title='Admin Panel')


@bp.route('/admin/user_management', methods=['GET', 'POST'])
@login_required
@admin_permission.require()
def user_management():
    users = User.query.all()

    return render_template('user_management.html', title='Admin Panel - User Management', users=[u for u in users])


@bp.route('/admin/user_management/edit_user/<user_id>', methods=['GET', 'POST'])
@login_required
@admin_permission.require()
def edit_user(user_id):
    form_user = User.query.filter_by(id=user_id).first()
    form = EditUserForm()

    daily_driver = False
    weekly_restaurant = False


    if form.validate_on_submit():
        form_user.username = form.username.data
        form_user.email = form.email.data
        form_user.first_name = form.first_name.data
        form_user.last_name = form.last_name.data
        form_user.is_disabled = form.is_disabled.data
        form_user.role = form.role.data

        if form.active.data:
            form_user.set_active()
        else:
            form_user.set_inactive()
            daily_driver, weekly_restaurant = form_user.check_active_drivers()

        if daily_driver:
            flash("Running Daily Driver Reports")
        if weekly_restaurant:
            flash("Running Weekly Restaurant Reports")


        db.session.commit()
        flash("{0}'s profile updated".format(form_user.username))
        return render_template('index.html', title="Admin Panel - Edit {0}'s profile".format(form_user.username), form=form)

    elif request.method == 'GET':
        form.username.data = form_user.username
        form.email.data = form_user.email
        form.first_name.data = form_user.first_name
        form.last_name.data = form_user.last_name
        form.is_disabled.data = form_user.is_disabled
        form.role.data = form_user.role
        form.active.data = form_user.active

    return render_template('index.html', title="Admin Panel - Edit {0}'s profile".format(form_user.username), form=form)


@bp.route('/admin/user_management/new_user', methods=['GET', 'POST'])
@login_required
@admin_permission.require()
def new_user():
    form = NewUserForm()
    if form.validate_on_submit():
        user = User()
        user.username = form.username.data
        user.email = form.email.data
        user.email_confirmed_at = datetime.utcnow()
        user.set_password(form.password2.data)
        user.first_name = form.first_name.data
        user.last_name = form.last_name.data
        user.role = form.role.data

        db.session.add(user)
        db.session.commit()
        flash("User {} created".format(form.username.data))

    elif request.method == 'GET':
        return render_template('index.html', title='New user', form=form)

    return render_template('index.html', title='Admin Panel - New user', form=form)

@bp.route('/admin/user_management/reset_password/<user_id>', methods=['GET', 'POST'])
@login_required
@admin_permission.require()
def admin_reset_password(user_id):
    user = User.query.filter_by(id=user_id).first()
    form = AdminResetPasswordForm()
    if form.validate_on_submit():
        user.set_password(form.password.data)
        db.session.commit()
        flash("User's password has been reset.")
        return redirect(url_for('main.user', username=user.username))
    return render_template('auth/reset_password.html', form=form, username=user.username)


@bp.route('/admin/order_management', methods=['GET', 'POST'])
@login_required
@admin_permission.require()
def order_management():
    page = request.args.get('page', 1, type=int)
    orders = Order.query.order_by(Order.datetime.desc()).paginate(page, 25, False)
    next_url = url_for('main.order_management', page=orders.next_num) if orders.has_next else None
    prev_url = url_for('main.order_management', page=orders.prev_num) if orders.has_prev else None
    return render_template('order_management.html', title='Admin Panel - Order Management', orders=orders.items, next_url=next_url, prev_url=prev_url)


@bp.route('/admin/order_management/edit_order/<order_id>', methods=['GET', 'POST'])
@login_required
@admin_permission.require()
def edit_order(order_id):
    order = Order.query.filter_by(id=order_id).first()
    form = EditOrderForm()
    driver = User.query.filter_by(id=order.user_id).first()

    daily_driver = False
    weekly_restaurant = False

    if form.validate_on_submit():
        order.date = form.date.data
        order.restaurant = Restaurant.query.filter_by(name=form.restaurant.data).first().id
        order.receipt = form.receipt.data
        order.customer_address = form.customer_address.data
        order.driver_paid_amount = form.driver_paid_amount.data
        order.total_restaurant_amount = form.total_restaurant_amount.data
        order.tips_amount = form.tips.data
        order.payment_type_id = PaymentType.query.filter_by(name=form.customer_payment_type.data).first().id
        order.additional_pickup = form.additional_pickup.data

        flash("Order {} updated".format(order.id))
        if form.last_order.data:
            driver.set_active()
        else:
            driver.set_inactive()
            daily_driver, weekly_restaurant = driver.check_active_drivers()

        if daily_driver:
            flash("Running Daily Driver Reports")
        if weekly_restaurant:
            flash("Running Weekly Restaurant Reports")

        db.session.commit()
        return redirect(url_for('main.order_management'))

    elif request.method == 'GET':
        form.date.data = order.date
        form.restaurant.data = order.restaurant()
        form.receipt.data = order.receipt
        form.customer_address.data = order.customer_address
        form.driver_paid_amount.data = order.driver_paid_amount
        form.total_restaurant_amount.data = order.total_restaurant_amount
        form.tips.data = order.tips_amount
        form.customer_payment_type.data = order.customer_payment_type()
        form.additional_pickup.data = order.additional_pickup
        form.last_order.data = False

    return render_template('index.html', title='Admin Panel - Edit Order', form=form)


@bp.route('/admin/restaurant_management', methods=['GET', 'POST'])
@login_required
@admin_permission.require()
def restaurant_management():
    restaurants = Restaurant.query.all()
    print([r for r in restaurants])
    return render_template('restaurant_management.html', title='Admin Panel - Restaurant Management', restaurants=[r for r in restaurants])


@bp.route('/admin/restaurant_management/edit_restaurant/<restaurant_id>', methods=['GET', 'POST'])
@login_required
@admin_permission.require()
def edit_restaurant(restaurant_id):
    restaurant = Restaurant.query.filter_by(id=restaurant_id).first()
    form = EditRestaurantForm()

    if form.validate_on_submit():
        restaurant.name = form.name.data
        restaurant.active = form.active.data
        restaurant.driver_fee = form.driver_fee.data
        restaurant.additional_driver_fee = form.additional_driver_fee.data
        restaurant.crave_fee = form.crave_fee.data
        restaurant.additional_crave_fee = form.additional_crave_fee.data
        restaurant.service_fee = form.service_fee.data
        restaurant.commission = form.commission.data
        restaurant.prepaid_commission = form.prepaid_commission.data
        restaurant.monthly_fee = form.monthly_fee.data
        restaurant.delivery_rebate = form.delivery_rebate.data

        restaurant.prepaid_only = form.prepaid_only.data
        # todo make these do something

        flash("Restaurant {} updated".format(restaurant.name))

        # if restaurant.active:
        #     restaurant.set_active()
        # else:
        #     restaurant.set_inactive()

        db.session.commit()

        return redirect(url_for('main.restaurant_management'))

    elif request.method == 'GET':
        form.name.data = restaurant.name
        form.active.data = restaurant.active
        form.driver_fee.data = restaurant.driver_fee
        form.additional_driver_fee.data = restaurant.additional_driver_fee
        form.crave_fee.data = restaurant.crave_fee
        form.additional_crave_fee.data = restaurant.additional_crave_fee
        form.service_fee.data = restaurant.service_fee
        form.commission.data = restaurant.commission
        form.prepaid_commission.data = restaurant.prepaid_commission
        form.monthly_fee.data = restaurant.monthly_fee
        form.delivery_rebate.data = restaurant.delivery_rebate
        form.prepaid_only.data = restaurant.prepaid_only
        form.active.data = restaurant.active

    return render_template('index.html', title='Admin Panel - Edit Order', form=form)


@bp.route('/admin/restaurant_management/new_restaurant', methods=['GET', 'POST'])
@login_required
@admin_permission.require()
def new_restaurant():
    form = NewRestaurantForm()

    if form.validate_on_submit():
        restaurant = Restaurant()
        restaurant.name = form.name.data
        restaurant.active = form.active.data
        restaurant.driver_fee = form.driver_fee.data
        restaurant.additional_driver_fee = form.additional_driver_fee.data
        restaurant.crave_fee = form.crave_fee.data
        restaurant.additional_crave_fee = form.additional_crave_fee.data
        restaurant.service_fee = form.service_fee.data
        restaurant.commission = form.commission.data
        restaurant.prepaid_commission = form.prepaid_commission.data
        restaurant.monthly_fee = form.monthly_fee.data
        restaurant.delivery_rebate = form.delivery_rebate.data
        restaurant.prepaid_only = form.prepaid_only.data
        db.session.add(restaurant)
        db.session.commit()
        flash("Restaurant: {} created".format(restaurant.name))
        return redirect(url_for('main.restaurant_management'))

    return render_template('index.html', title='Admin Panel - New Restaurant', form=form)